/// <reference types="node" />
import { Message, PermissionFlags, Snowflake } from 'discord.js';
import { SilentClient } from '../../';
import { ICommandOptions } from './typings/ICommandOptions';
import { IOnlyOptions } from './typings/IOnlyOptions';
import { IVerify } from './typings/IVerify';
import Timer = NodeJS.Timer;
/**
 * The Command class to create and initialise new commands
 * @abstract
 */
export declare abstract class Command<S extends SilentClient = SilentClient> {
    /**
     * The base name of the command. This is used in error message about the command and to find it.
     * @property {String} commandName - The name of the command
     * @public
     */
    commandName: string;
    /**
     * An array of aliases for the command.
     * @property {String[]} alias - An array of command aliases
     * @public
     */
    alias: string[];
    /**
     * Restricting the command usage to specific regions like guild, dm, etc
     * @property {IOnlyOptions} only - Object of restricting attributes
     * @public
     */
    only: IOnlyOptions;
    /**
     * Ratelimit information
     * @namespace
     * @property {Object} ratelimit - Holding information about the ratelimit options
     * @property {Number} ratelimit.times - How often a command can be used within x interval
     * @property {Number} ratelimit.timeout - The interval between clearing the data
     * @property {Boolean} ratelimit.all - Weather to include owners in the ratelimit
     * @public
     */
    ratelimit: {
        times: number;
        timeout: number;
        all: boolean;
    };
    /**
     * Additional verify function to inhibit command usage.
     * @property {?Function} extendedVerify - An extending verify function
     * @public
     */
    extendedVerify: (msg: Message, client: S) => IVerify | Promise<IVerify>;
    /**
     * Restricting command to users who do not have set permissions.
     * @property {PermissionFlags[]} userPermissions - Holding required permissions for users to have
     * @public
     */
    userPermissions: PermissionFlags[];
    /**
     * Restricting command to specific roles or users
     * @property {{roles: Snowflake[], users: Snowflake[]}} restricted - information about how is able to use a cmd
     * @public
     */
    restricted: {
        roles: Snowflake[];
        users: Snowflake[];
    };
    /**
     * The description of the command
     * @property {string} desc - Description of command
     */
    desc: string;
    /**
     * Example usage of the command
     * @property {string} usage - Usage of command
     */
    usage: string;
    /**
     * Restricting the command to the owner
     * @property {boolean} ownerOnly
     */
    ownerOnly: boolean;
    /**
     * Restricting the command to NSFW channels
     * @property {boolean} nsfw
     */
    nsfw: boolean;
    /**
     * Setting a minimum role (in hierarchy) for command execution
     * @property {string|Snowflake} minRole
     */
    minRole: string | Snowflake;
    /**
     * Hide the command from the default help message
     * @property {boolean} hidden
     */
    hidden: boolean;
    /**
     * @property {String} _className
     * @private
     */
    _className: string;
    /**
     * @property {Object} client - The client instance
     * @public
     */
    client: S;
    /**
     * @namespace ThrottleObject
     * @property {Number} ThrottleObject.start - Start of the throttling process
     * @property {Number} ThrottleObject.uses - Command uses since start
     * @property {Timer} ThrottleObject.timeout - The timeout function to delete this object after the interval
     */
    /**
     * Map of current throttles
     * @property {Map<Snowflake, ThrottleObject>} - Storage of the ratelimit
     * @protected
     */
    throttled: Map<Snowflake, {
        start: number;
        uses: number;
        timeout: Timer;
    }>;
    protected constructor(options?: ICommandOptions);
    /**
     * Initializes the command to set all needed properties and check the configurations for errors
     * @param client - The client object
     * @returns {Command}
     * @private
     */
    _init(client: S): this;
    /**
     * Get the current status of a user's ratelimit
     * @param userID {Snowflake} The user ID of the person to limit
     * @returns {ThrottleObject}
     * @public
     */
    throttle(userID: Snowflake): {
        start: number;
        uses: number;
        timeout: Timer;
    };
    /**
     * This is used to verify that the command is a valid command and all requirements for it are met
     * You can customize this with the @VerifyExtend decorator
     * @param msg {Message}
     * @param [client] {Object} The client object
     * @returns {Promise<Boolean>}
     * @public
     */
    verify(msg: Message, client?: S): Promise<IVerify>;
    /**
     * Run the command
     * @param values {*[]}
     * @abstract
     * @public
     */
    abstract run(...values: any[]): Promise<IVerify>;
}
