"use strict";
/*
 * Copyright (c) 2019., Charlie Tiehm - admin@tiehm.me
 */
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ArgumentMetadata_1 = require("./ArgumentMetadata");
const Logger_1 = require("./Logger");
class Listener {
    /**
     *
     * @param client {SilentClient}
     * @param [waitForImplementation=true] {Boolean}
     */
    constructor(client, waitForImplementation = true) {
        this.client = client;
        this.wait = waitForImplementation;
    }
    /**
     * Handle a message event
     * @param msg {Message}
     * @async
     * @public
     */
    handleMessageEvent(msg) {
        return __awaiter(this, void 0, void 0, function* () {
            this.logger.debug(`Handling incoming message with content: ${msg.content}`);
            // Runs the custom implemented message event
            // Option to wait for the return value of the custom event
            // Returns and blocks commands if the custom event returns
            // A falsie value or Promise resolving to such
            if (this.client.events.get('message')) {
                this.logger.debug('Executing custom message event');
                let ranEvent = this.client.events.get('message').run(msg);
                if (this.wait) {
                    if (ranEvent.constructor.name === 'Promise') {
                        ranEvent = yield ranEvent;
                    }
                    if (!ranEvent)
                        return false;
                }
            }
            const command = this._findCommandInMessage(msg);
            if (!command && this.client.commandNotFoundError) {
                return msg.reply(this.client.commandNotFoundError);
            }
            if (!command)
                return;
            this.logger.debug(`Message is a command, command found: ${command.commandName}`);
            let verify = command.verify(msg, this.client);
            // @ts-ignore
            if (verify.constructor.name === 'Promise')
                verify = yield verify;
            if (!verify)
                return this.logger.debug(`Stopped executing command ${command.commandName} as verify failed.`);
            const args = msg.content.trim().split(/ +/g).slice(1);
            const properties = ArgumentMetadata_1.argumentStore
                .filter(value => value.name === command._className.toLowerCase())
                .sort((a, b) => a.index - b.index).map(value => value.value(msg));
            if (command.ratelimit) {
                const throttle = command.throttle(msg.author.id);
                if (throttle)
                    throttle.uses += 1;
            }
            let executed;
            try {
                if (properties.length === 0) {
                    this.logger.debug('Running command with default arguments');
                    executed = yield command.run(msg, args);
                }
                else {
                    this.logger.debug('Running command with decorator arguments');
                    executed = yield command.run.apply(command, properties);
                }
                if (executed) {
                    if (executed.channel)
                        yield msg.reply('This command can not be used in this channel.');
                    else if (executed.dm)
                        yield msg.reply('This command can only be used within DMs.');
                    else if (executed.guild)
                        yield msg.reply('This command can only be used within guilds.');
                    else if (executed.nsfw)
                        yield msg.reply('This command can only be used in NSFW channels.');
                    else if (executed.owner)
                        yield msg.reply('This command can only be used by the Bot Owner.');
                    else if (executed.permission) {
                        yield msg.reply('You do not have enough permissions to use this command.');
                    }
                    else if (executed.restricted)
                        yield msg.reply('You can not use this command.');
                    else if (executed.role)
                        yield msg.reply('Your role is too low to use this command.');
                    else if (executed.syntax) {
                        // tslint:disable-next-line
                        yield msg.reply(`You used this command in the wrong way.\nSyntax: \`${this.client.defaultPrefix}${command.commandName} ${command.usage}\`.`);
                    }
                }
            }
            catch (e) {
                this.logger.error(`Error on command execution: ${command.commandName}`);
                this.logger.error(e);
            }
        });
    }
    /**
     * Find a command in a message
     * @param msg {Message}
     * @private
     * @returns {Command}
     */
    _findCommandInMessage(msg) {
        let command = null;
        if (msg.isMemberMentioned(this.client.user) &&
            this.client.mentionPrefix && (msg.content.startsWith(`<@${this.client.user.id}>`) ||
            msg.content.startsWith(`<@!${this.client.user.id}>`))) {
            const foundMsgCmd = msg.content.split(/ +/g)[1];
            if (this.client.commands.get(foundMsgCmd))
                command = this.client.commands.get(foundMsgCmd);
            if (this.client.commands.find(c => c.alias && c.alias.includes(foundMsgCmd))) {
                command = this.client.commands.find(c => c.alias.includes(foundMsgCmd));
            }
        }
        else if (typeof this.client.prefix === 'string') {
            if (msg.content.startsWith(this.client.prefix)) {
                const foundMessageCmd = msg.content.slice(this.client.prefix.length).trim().split(/ +/g)[0].toLowerCase();
                command = this.client.commands.get(foundMessageCmd) ||
                    this.client.commands.find((c) => {
                        if (c.alias)
                            return c.alias.includes(foundMessageCmd);
                        return false;
                    });
            }
        }
        else if (Array.isArray(this.client.prefix)) {
            for (const prefix of this.client.prefix) {
                if (!msg.content.startsWith(prefix))
                    continue;
                const foundMsgCmd = msg.content.slice(prefix.length).trim().split(/ +/g)[0].toLowerCase();
                if (this.client.commands.get(foundMsgCmd))
                    command = this.client.commands.get(foundMsgCmd);
                if (this.client.commands.find(c => c.alias && c.alias.includes(foundMsgCmd))) {
                    command = this.client.commands.find(c => c.alias && c.alias.includes(foundMsgCmd));
                }
            }
        }
        return command;
    }
}
__decorate([
    Logger_1.logger()
], Listener.prototype, "logger", void 0);
exports.Listener = Listener;
//# sourceMappingURL=Listener.js.map