"use strict";
/*
 * Copyright (c) 2019., Charlie Tiehm - admin@tiehm.me
 */
Object.defineProperty(exports, "__esModule", { value: true });
const chalk_1 = require("chalk");
const moment = require("moment");
const loggers = new Map();
/**
 * The logger
 */
class Logger {
    /**
     * Outputs the message to the console
     * @param text {String} The message to log
     * @static
     */
    static write(text) {
        process.stdout.write(`${text}\n`);
    }
    /**
     * @param name {String} The name displayed when logging something
     * @returns {Logger}
     */
    constructor(name) {
        this.name = name.toUpperCase();
        return this;
    }
    /**
     * Log something without any format, this equals a console.log
     * @param msg {String} The message to log
     */
    rawLog(msg) {
        Logger.write(msg);
    }
    /**
     * Logs the error
     * @param err {String|Error} The message or error to log
     */
    error(err) {
        if (typeof err === 'string') {
            const msg = this.format() + chalk_1.default.red(err);
            Logger.write(msg);
        }
        else {
            const [name, ...stack] = err.stack.split('\n');
            const msg = `${this.format() + chalk_1.default.red(name)}\n${chalk_1.default.gray(stack.join('\n'))}`;
            Logger.write(msg);
        }
    }
    /**
     * Log the message if the log level is not restricted to errors only
     * @param text {String} The message to log
     */
    warn(text) {
        if (process.env.LOGLEVEL !== 'ERROR') {
            const msg = this.format() + chalk_1.default.yellow(text);
            Logger.write(msg);
        }
    }
    /**
     * Log the message if the log level is not restricted to errors or warns only
     * @param text {String} The message to log
     */
    log(text) {
        if (process.env.LOGLEVEL === 'DEBUG' || process.env.LOGLEVEL === 'ALL') {
            const msg = this.format() + text;
            Logger.write(msg);
        }
    }
    /**
     * Log the message if the log level is in debug mode
     * @param text {String} The message to log
     */
    debug(text) {
        if (process.env.LOGLEVEL === 'DEBUG') {
            const msg = this.format() + chalk_1.default.magenta(text);
            Logger.write(msg);
        }
    }
    /**
     * Format a message to include the log name and the current time
     * @returns {String}
     */
    format() {
        return chalk_1.default.gray(`[${moment().format('HH:mm:ss')}] [${this.name}] `);
    }
}
exports.Logger = Logger;
function logger(type) {
    return function (target, propertyKey) {
        // tslint:disable-next-line
        if (!type)
            type = target.constructor.name;
        // tslint:disable-next-line
        type = type.toUpperCase();
        const log = loggers.get(type) || new Logger(type);
        Object.defineProperty(target, propertyKey, {
            value: log
        });
    };
}
exports.logger = logger;
//# sourceMappingURL=Logger.js.map